from django.conf import settings
from django.db import models
from django.utils.importlib import import_module
from django.utils.encoding import smart_str, smart_unicode

from m2secret import Secret


class EncryptedField(Exception):
    pass


class AESField(models.TextField):

    description = 'A field that uses AES encryption.'
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        self.aes_prefix = kwargs.pop('aes_prefix', 'aes:')
        if not self.aes_prefix:
            raise ValueError('AES Prefix cannot be null.')
        self.aes_method = getattr(settings, 'AES_METHOD', 'aesfield.default')
        self.aes_key = kwargs.pop('aes_key', '')
        super(AESField, self).__init__(*args, **kwargs)

    def get_aes_key(self):
        result = import_module(self.aes_method).lookup(self.aes_key)
        if len(result) < 10:
            raise ValueError('Passphrase cannot be less than 10 chars.')
        return result

    def get_prep_lookup(self, type, value):
        raise EncryptedField('You cannot do lookups on an encrypted field.')

    def get_db_prep_lookup(self, *args, **kw):
        raise EncryptedField('You cannot do lookups on an encrypted field.')

    def get_db_prep_value(self, value, connection, prepared=False):
        if not prepared and value:
            return self.aes_prefix + self._encrypt(value)
        return value

    def _encrypt(self, value):
        secret = Secret()
        secret.encrypt(smart_str(value), self.get_aes_key())
        return secret.serialize()

    def to_python(self, value):
        if not value or not value.startswith(self.aes_prefix):
            return value
        return self._decrypt(value[len(self.aes_prefix):])

    def _decrypt(self, value):
        secret = Secret()
        secret.deserialize(value)
        return smart_unicode(secret.decrypt(self.get_aes_key()))


# South support.
try:
    from south.modelsinspector import add_introspection_rules
except ImportError:
    pass
else:
    add_introspection_rules(rules=[(
        (AESField,),
        [],
        {
            'aes_prefix': ['aes_prefix', {'default': 'aes:'}],
            'aes_key': ['aes_key', {'default': ''}],
        },
    )], patterns=['^aesfield\.field\.AESField'])



# -*- coding: utf8 -*-
import os
import tempfile

from django.conf import settings

minimal = {
    'DATABASES': {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': 'mydatabase'
        }
    },
    'AES_KEYS': {'default': os.path.join(os.path.dirname(__file__), 'sample.key')}
}

if not settings.configured:
    settings.configure(**minimal)

from django.db import models
from django.test import TestCase

from nose.tools import eq_

from .field import AESField, EncryptedField
from .management.commands.generate_aes_keys import Command, CommandError


class TestModel(models.Model):
    key = AESField(max_length=255)


class TestBasic(TestCase):

    def test_lookup(self):
        with self.assertRaises(EncryptedField):
            TestModel.objects.filter(key='asd')

    def test_no_prefix(self):
        with self.assertRaises(ValueError):
            AESField(aes_prefix='')

    def test_get_key(self):
        key = 'some-super-secret-key'
        fn = tempfile.NamedTemporaryFile()
        fn.write(key)
        fn.flush()
        with self.settings(AES_KEYS={'default': fn.name}):
            assert AESField().get_aes_key() == 'some-super-secret-key'

    def test_generate_fails(self):
        with self.assertRaises(CommandError):
            Command().handle()

    def test_encrypt_decrypt(self):
        t = TestModel()
        f = t._meta.get_field('key')
        for k in ['foo',
                  '1234567890123456',
                  '12345678901234567',
                  u'ራ'.encode('utf8')]:
            eq_(k.decode('utf8'), f._decrypt(f._encrypt(k)))

    def test_not_encoded(self):
        t = TestModel()
        f = t._meta.get_field('key')
        k = u'ራ'
        eq_(k, f._decrypt(f._encrypt(k)))



import os
from setuptools import setup


setup(
    name='django-aesfield',
    version='0.3.0',
    description='Django Model Field that supports AES encryption',
    long_description=open('README.rst').read(),
    author='Andy McKay',
    author_email='andym@mozilla.com',
    license='BSD',
    url='https://github.com/andymckay/django-aesfield',
    include_package_data=True,
    zip_safe=False,
    packages=['aesfield',
              'aesfield/management',
              'aesfield/management/commands'],
    install_requires=['M2Crypto>=0.18',
                      'Django>=1.3',
                      'm2secret>=0.1.1'],
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Framework :: Django'
        ],
    )
